#ifndef __FSM_H__
#define __FSM_H__
namespace AxisG2Selector_ns
{

#include "Types.h"
#include "Targets.h"

class AxisG2Selector;

typedef enum {
        FSM_ERROR,
        FSM_NOTINITED,
        FSM_WAIT,
        FSM_START,
        FSM_MOVING,
	FSM_DONE,
	FSM_WAIT_TARGET,
	FSM_TARGET_DONE,
	FSM_TARGET_ERROR,
	FSM_WAIT_AXISINIT
} TrajectoryFSM_t;

const char * const TrajectoryFSMName[] = {
        "FSM_ERROR",
        "FSM_NOTINITED",
        "FSM_WAIT",
        "FSM_START",
        "FSM_MOVING",
	"FSM_DONE",
	"FSM_WAIT_TARGET",
	"FSM_TARGET_DONE",
	"FSM_TARGET_ERROR",
	"FSM_WAIT_AXISINIT"
};

typedef enum {
        FSMT_NOP,
        FSMT_ABORT,
        FSMT_GOTO,
	FSMT_AXISINIT
} TrajectoryFSMTransitions_t;

const char * const TrajectoryFSMTransitions[] = {
        "FSMT_NOP: no operation",
        "FSMT_ABORT: abort FSM job",
        "FSMT_GOTO: goto target",
        "FSMT_AXISINIT: sources axis init"
};


class TrajectoryFSM {

private:
	AxisG2Selector *		_dev;
	TrajectoryFSM_t 		_state;
	TrajectoryFSMTransitions_t 	_tr;
	TrajectoryFSMTransitions_t 	_working_tr;
	Target				_working_trg;
	int 				_cnt;

public:
	TrajectoryFSM(AxisG2Selector * dev);
	~TrajectoryFSM();

	void		fsm(TrajectoryFSMTransitions_t tr, Target trg);           // proceed with FSM
	void		init() 		{ _state = FSM_WAIT;  _tr = FSMT_NOP; };
	TrajectoryFSM_t state()		{ return _state; };
        string          status()        { switch(_state){
                                                        case FSM_ERROR:
                                                        case FSM_NOTINITED:
                                                        case FSM_WAIT:
                                                        case FSM_START:
                                                        case FSM_MOVING:
                                                        case FSM_DONE:
							case FSM_WAIT_TARGET:
							case FSM_TARGET_DONE:
							case FSM_TARGET_ERROR:
							case FSM_WAIT_AXISINIT:
                                                             return TrajectoryFSMName[_state];
                                                             break;
                                                        default: {
                                                             std::stringstream ss;
                                                             ss << "-?- [state= " << _state << " ]";
                                                             return ss.str();
                                                            }
                                                        break;
                                                       }
                                                       return "-?-";
                                         };

};



} //namespace
#endif /* __FSM_H__ */
